@@ -28,10 +28,11 @@ TODO:
 
 import os
 import sys
+from contextlib import suppress
 from typing import (BinaryIO, Iterable, Iterator, KeysView, List,
                     MutableMapping, Optional, Tuple, Union, overload)
 
-from dulwich.file import GitFile
+from .file import GitFile
 
 SENTINEL = object()
 
@@ -641,13 +642,11 @@ def _find_git_in_win_reg():
         )
 
     for key in (winreg.HKEY_CURRENT_USER, winreg.HKEY_LOCAL_MACHINE):  # type: ignore
-        try:
+        with suppress(OSError):
             with winreg.OpenKey(key, subkey) as k:  # type: ignore
                 val, typ = winreg.QueryValueEx(k, "InstallLocation")  # type: ignore
                 if typ == winreg.REG_SZ:  # type: ignore
                     yield val
-        except OSError:
-            pass
 
 
 # There is no set standard for system config dirs on windows. We try the
