@@ -22,63 +22,33 @@
 
 """Git object store interfaces and implementation."""
 
-from contextlib import suppress
-from io import BytesIO
 import os
 import stat
 import sys
 import warnings
-
-from typing import Callable, Dict, List, Optional, Tuple, Iterator, Set, Iterable, Sequence, cast
+from contextlib import suppress
+from io import BytesIO
+from typing import (Callable, Dict, Iterable, Iterator, List, Optional,
+                    Sequence, Set, Tuple, cast)
 
 try:
     from typing import Protocol
 except ImportError:  # python << 3.8
     from typing_extensions import Protocol  # type: ignore
 
-from dulwich.errors import (
-    NotTreeError,
-)
-from dulwich.file import GitFile
-from dulwich.objects import (
-    ObjectID,
-    Commit,
-    ShaFile,
-    Tag,
-    Tree,
-    Blob,
-    ZERO_SHA,
-    hex_to_sha,
-    sha_to_hex,
-    hex_to_filename,
-    S_ISGITLINK,
-    TreeEntry,
-    object_class,
-    valid_hexsha,
-)
-from dulwich.pack import (
-    ObjectContainer,
-    Pack,
-    PackData,
-    PackHint,
-    PackInflater,
-    PackFileDisappeared,
-    UnpackedObject,
-    extend_pack,
-    load_pack_index_file,
-    iter_sha1,
-    full_unpacked_object,
-    generate_unpacked_objects,
-    pack_objects_to_data,
-    write_pack_index,
-    write_pack_data,
-    PackIndexer,
-    PackStreamCopier,
-    PackedObjectContainer,
-    PACK_SPOOL_FILE_MAX_SIZE,
-)
-from dulwich.protocol import DEPTH_INFINITE
-from dulwich.refs import PEELED_TAG_SUFFIX, Ref
+from .errors import NotTreeError
+from .file import GitFile
+from .objects import (S_ISGITLINK, ZERO_SHA, Blob, Commit, ObjectID, ShaFile,
+                      Tag, Tree, TreeEntry, hex_to_filename, hex_to_sha,
+                      object_class, sha_to_hex, valid_hexsha)
+from .pack import (PACK_SPOOL_FILE_MAX_SIZE, ObjectContainer, Pack, PackData,
+                   PackedObjectContainer, PackFileDisappeared, PackHint,
+                   PackIndexer, PackInflater, PackStreamCopier, UnpackedObject,
+                   extend_pack, full_unpacked_object,
+                   generate_unpacked_objects, iter_sha1, load_pack_index_file,
+                   pack_objects_to_data, write_pack_data, write_pack_index)
+from .protocol import DEPTH_INFINITE
+from .refs import PEELED_TAG_SUFFIX, Ref
 
 INFODIR = "info"
 PACKDIR = "pack"
@@ -188,7 +158,7 @@ class BaseObjectStore:
             (oldpath, newpath), (oldmode, newmode), (oldsha, newsha)
         """
 
-        from dulwich.diff_tree import tree_changes
+        from .diff_tree import tree_changes
         for change in tree_changes(
             self,
             source,
@@ -228,6 +198,42 @@ class BaseObjectStore:
                 if not allow_missing:
                     raise
 
+    def find_missing_objects(
+        self,
+        haves,
+        wants,
+        shallow=None,
+        progress=None,
+        get_tagged=None,
+        get_parents=lambda commit: commit.parents,
+    ):
+        """Find the missing objects required for a set of revisions.
+
+        Args:
+          haves: Iterable over SHAs already in common.
+          wants: Iterable over SHAs of objects to fetch.
+          shallow: Set of shallow commit SHA1s to skip
+          progress: Simple progress function that will be called with
+            updated progress strings.
+          get_tagged: Function that returns a dict of pointed-to sha ->
+            tag sha for including tags.
+          get_parents: Optional function for getting the parents of a
+            commit.
+        Returns: Iterator over (sha, path) pairs.
+        """
+        warnings.warn(
+            'Please use MissingObjectFinder(store)', DeprecationWarning)
+        finder = MissingObjectFinder(
+            self,
+            haves=haves,
+            wants=wants,
+            shallow=shallow,
+            progress=progress,
+            get_tagged=get_tagged,
+            get_parents=get_parents,
+        )
+        return iter(finder)
+
     def find_common_revisions(self, graphwalker):
         """Find which revisions this store has in common using graphwalker.
 
@@ -1039,6 +1045,16 @@ class MemoryObjectStore(BaseObjectStore):
 
         return f, commit, abort
 
+    def add_pack_data(self, count: int, unpacked_objects: Iterator[UnpackedObject], progress=None) -> None:
+        """Add pack data to this object store.
+
+        Args:
+          count: Number of items to add
+          pack_data: Iterator over pack data tuples
+        """
+        for unpacked_object in unpacked_objects:
+            self.add_object(unpacked_object.sha_file())
+
     def add_thin_pack(self, read_all, read_some, progress=None):
         """Add a new thin pack to this object store.
 
