@@ -22,27 +22,15 @@
 """Ref handling.
 
 """
-from contextlib import suppress
 import os
-from typing import Dict, Optional
 import warnings
+from contextlib import suppress
+from typing import Dict, Optional
 
-from dulwich.errors import (
-    PackedRefsException,
-    RefFormatError,
-)
-from dulwich.objects import (
-    git_line,
-    valid_hexsha,
-    ZERO_SHA,
-    Tag,
-    ObjectID,
-)
-from dulwich.pack import ObjectContainer
-from dulwich.file import (
-    GitFile,
-    ensure_dir_exists,
-)
+from .errors import PackedRefsException, RefFormatError
+from .file import GitFile, ensure_dir_exists
+from .objects import ZERO_SHA, ObjectID, Tag, git_line, valid_hexsha
+from .pack import ObjectContainer
 
 Ref = bytes
 
@@ -50,6 +38,7 @@ HEADREF = b"HEAD"
 SYMREF = b"ref: "
 LOCAL_BRANCH_PREFIX = b"refs/heads/"
 LOCAL_TAG_PREFIX = b"refs/tags/"
+LOCAL_REMOTE_PREFIX = b"refs/remotes/"
 BAD_REF_CHARS = set(b"\177 ~^:?*[")
 PEELED_TAG_SUFFIX = b"^{}"
 
@@ -745,10 +734,8 @@ class DiskRefsContainer(RefsContainer):
                 # remove any loose refs pointing to this one -- please
                 # note that this bypasses remove_if_equals as we don't
                 # want to affect packed refs in here
-                try:
+                with suppress(OSError):
                     os.remove(self.refpath(ref))
-                except OSError:
-                    pass
 
                 if target is not None:
                     packed_refs[ref] = target
@@ -1158,7 +1145,7 @@ def read_info_refs(f):
 def write_info_refs(refs, store: ObjectContainer):
     """Generate info refs."""
     # TODO: Avoid recursive import :(
-    from dulwich.object_store import peel_sha
+    from .object_store import peel_sha
     for name, sha in sorted(refs.items()):
         # get_refs() includes HEAD as a special case, but we don't want to
         # advertise it
@@ -1286,7 +1273,7 @@ def _import_remote_refs(
 
 def serialize_refs(store, refs):
     # TODO: Avoid recursive import :(
-    from dulwich.object_store import peel_sha
+    from .object_store import peel_sha
     ret = {}
     for ref, sha in refs.items():
         try:
