@@ -23,49 +23,23 @@
 # TODO: Round-trip parse-serialize-parse and serialize-parse-serialize tests.
 
 
-from io import BytesIO
 import datetime
-from itertools import (
-    permutations,
-)
 import os
 import stat
 from contextlib import contextmanager
+from io import BytesIO
+from itertools import permutations
 
-from dulwich.errors import (
-    ObjectFormatException,
-)
-from dulwich.objects import (
-    Blob,
-    Tree,
-    Commit,
-    ShaFile,
-    Tag,
-    TreeEntry,
-    format_timezone,
-    hex_to_sha,
-    sha_to_hex,
-    hex_to_filename,
-    check_hexsha,
-    check_identity,
-    object_class,
-    parse_timezone,
-    pretty_format_tree_entry,
-    parse_tree,
-    _parse_tree_py,
-    sorted_tree_items,
-    _sorted_tree_items_py,
-    MAX_TIME,
-)
-from dulwich.tests import (
-    TestCase,
-)
-from dulwich.tests.utils import (
-    make_commit,
-    make_object,
-    functest_builder,
-    ext_functest_builder,
-)
+from dulwich.tests import TestCase
+
+from ..errors import ObjectFormatException
+from ..objects import (MAX_TIME, Blob, Commit, ShaFile, Tag, Tree, TreeEntry,
+                       _parse_tree_py, _sorted_tree_items_py, check_hexsha,
+                       check_identity, format_timezone, hex_to_filename,
+                       hex_to_sha, object_class, parse_timezone, parse_tree,
+                       pretty_format_tree_entry, sha_to_hex, sorted_tree_items)
+from .utils import (ext_functest_builder, functest_builder, make_commit,
+                    make_object)
 
 a_sha = b"6f670c0fb53f9463760b7295fbb814e965fb20c8"
 b_sha = b"2969be3e8ee1c0222396a5611407e4769f14e54b"
@@ -624,7 +598,7 @@ class CommitParseTests(ShaFileCheckTests):
 
     def test_custom(self):
         c = Commit.from_string(self.make_commit_text(extra={b"extra-field": b"data"}))
-        self.assertEqual([(b"extra-field", b"data")], c.extra)
+        self.assertEqual([(b"extra-field", b"data")], c._extra)
 
     def test_encoding(self):
         c = Commit.from_string(self.make_commit_text(encoding=b"UTF-8"))
@@ -757,7 +731,7 @@ foo
 """
         )
         self.assertEqual(b"foo\n", c.message)
-        self.assertEqual([], c.extra)
+        self.assertEqual([], c._extra)
         self.assertEqual(
             b"""-----BEGIN PGP SIGNATURE-----
 Version: GnuPG v1
@@ -801,7 +775,7 @@ gpgsig -----BEGIN PGP SIGNATURE-----
 3.3.0 version bump and docs
 """
         )
-        self.assertEqual([], c.extra)
+        self.assertEqual([], c._extra)
         self.assertEqual(
             b"""\
 -----BEGIN PGP SIGNATURE-----
@@ -980,7 +954,7 @@ class TreeTests(ShaFileCheckTests):
     def test_iter(self):
         t = Tree()
         t[b"foo"] = (0o100644, a_sha)
-        self.assertEqual(set([b"foo"]), set(t))
+        self.assertEqual({b"foo"}, set(t))
 
 
 class TagSerializeTests(TestCase):
@@ -1143,7 +1117,7 @@ class TagParseTests(ShaFileCheckTests):
 
     def test_check_tag_with_overflow_time(self):
         """Date with overflow should raise an ObjectFormatException when checked"""
-        author = "Some Dude <some@dude.org> %s +0000" % (MAX_TIME + 1,)
+        author = "Some Dude <some@dude.org> {} +0000".format(MAX_TIME + 1)
         tag = Tag.from_string(self.make_tag_text(tagger=(author.encode())))
         with self.assertRaises(ObjectFormatException):
             tag.check()
@@ -1301,14 +1275,14 @@ class TimezoneTests(TestCase):
         self.assertEqual(b"-0440", format_timezone(int(((-4 * 60) - 40) * 60)))
 
     def test_format_timezone_double_negative(self):
-        self.assertEqual(b"--700", format_timezone(int(((7 * 60)) * 60), True))
+        self.assertEqual(b"--700", format_timezone(int((7 * 60) * 60), True))
 
     def test_parse_timezone_pdt_half(self):
         self.assertEqual((((-4 * 60) - 40) * 60, False), parse_timezone(b"-0440"))
 
     def test_parse_timezone_double_negative(self):
-        self.assertEqual((int(((7 * 60)) * 60), False), parse_timezone(b"+700"))
-        self.assertEqual((int(((7 * 60)) * 60), True), parse_timezone(b"--700"))
+        self.assertEqual((int((7 * 60) * 60), False), parse_timezone(b"+700"))
+        self.assertEqual((int((7 * 60) * 60), True), parse_timezone(b"--700"))
 
 
 class ShaFileCopyTests(TestCase):
